//-----------------------------------------------------------------------
// A small TSP provider DLL sample.
//-----------------------------------------------------------------------
//
// Written by J-F. LAROCHE  and E. NOULARD
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
//
// -----------------------------------------------------------------------
//
// Project   : TSP
// Component : DLL Provider
// 
// -----------------------------------------------------------------------
// 
// Purpose   : A small TSP provider in the form of Windows DLL
//             Allow to define the number of symbol, the frequency,
//             adding symbol (only double are currently supported), 
//             and running the provider.
// 
// -----------------------------------------------------------------------


#include <tsp_sys_headers.h>    // platform independant data types definition
#include <tsp_glu.h>            // TSP GLU object definition and API
#include <tsp_ringbuf.h>
#include <tsp_datapool.h>       // TSP provider datapool API
#include <tsp_common.h>         // TSP common structure manipulation API

#include "tsp_provider_init.h"

#include "tsp_provider_dll.h"

// Some static needed for the dll
static TSP_sample_symbol_info_list_t X_SSI_list;

static int32_t nb_symbols = 0;
static int32_t nb_max_symbols = 0;
static int32_t max_symbol_size=0;

static GLU_handle_t *GLU_stub = NULL;
static glu_item_t   item;


// ---------------------------------------------------------------------------------------------------
// STUB_GLU_thread function
// ---------------------------------------------------------------------------------------------------
void* STUB_GLU_thread(void* athis)
{
    GLU_handle_t* cthis = (GLU_handle_t*) athis;
    return cthis;
}


// ---------------------------------------------------------------------------------------------------
// STUB_GLU_init function
// ---------------------------------------------------------------------------------------------------
int STUB_GLU_init(GLU_handle_t* cthis, int fallback_argc, char* fallback_argv[])
{
    return TRUE;
}


// ---------------------------------------------------------------------------------------------------
// STUB_GLU_get_ssi_list function
// ---------------------------------------------------------------------------------------------------
int STUB_GLU_get_ssi_list(GLU_handle_t*                  h_glu,
                          TSP_sample_symbol_info_list_t* symbol_list)
{
    symbol_list->TSP_sample_symbol_info_list_t_len = X_SSI_list.TSP_sample_symbol_info_list_t_len;
    symbol_list->TSP_sample_symbol_info_list_t_val = X_SSI_list.TSP_sample_symbol_info_list_t_val;

    return TRUE;
}


// ---------------------------------------------------------------------------------------------------
// STUB_GLU_get_ssei_list_fromPGI function
// ---------------------------------------------------------------------------------------------------

int32_t STUB_GLU_get_ssei_list_fromPGI(struct GLU_handle_t*                     cthis, 
			                           int32_t*                                 pgis,
                                       int32_t                                  pgis_len, 
			                           TSP_sample_symbol_extended_info_list_t*  SSEI_list)
{
    int32_t ii;
    int32_t ret = TSP_STATUS_OK;

    // Store all global indexes into list including NOT FOUND ones
    for (ii = 0 ; ii < pgis_len ; ++ii) {
        STRACE_DEBUG(("Getting extended info for PGI <%d>",pgis[ii]));
        if (-1 != pgis[ii]) {
            SSEI_list->TSP_sample_symbol_extended_info_list_t_val[ii].provider_global_index = pgis[ii]; 

            if (1000 > pgis[ii]) {
                SSEI_list->TSP_sample_symbol_extended_info_list_t_val[ii].info.TSP_extended_info_list_t_len = 0;
                SSEI_list->TSP_sample_symbol_extended_info_list_t_val[ii].info.TSP_extended_info_list_t_val = NULL;
            }
            else {
                switch (pgis[ii]) {
	                case 1000:
                        TSP_EIList_initialize(&(SSEI_list->TSP_sample_symbol_extended_info_list_t_val[ii].info),1);
                        TSP_EI_update(&(SSEI_list->TSP_sample_symbol_extended_info_list_t_val[ii].info.TSP_extended_info_list_t_val[0]), "unit", "a");
                        break; 

                    case 1001:
                        TSP_EIList_initialize(&(SSEI_list->TSP_sample_symbol_extended_info_list_t_val[ii].info),1);
                        TSP_EI_update(&(SSEI_list->TSP_sample_symbol_extended_info_list_t_val[ii].info.TSP_extended_info_list_t_val[0]), "unit", "b");
                        break; 

                    case 1002:
                        TSP_EIList_initialize(&(SSEI_list->TSP_sample_symbol_extended_info_list_t_val[ii].info),1);
                        TSP_EI_update(&(SSEI_list->TSP_sample_symbol_extended_info_list_t_val[ii].info.TSP_extended_info_list_t_val[0]), "unit", "c");
                        break; 

                    case 1003:
                        TSP_EIList_initialize(&(SSEI_list->TSP_sample_symbol_extended_info_list_t_val[ii].info),1);
                        TSP_EI_update(&(SSEI_list->TSP_sample_symbol_extended_info_list_t_val[ii].info.TSP_extended_info_list_t_val[0]), "unit", "d");
                        break; 

                    case 1004:
                        TSP_EIList_initialize(&(SSEI_list->TSP_sample_symbol_extended_info_list_t_val[ii].info),1);
                        TSP_EI_update(&(SSEI_list->TSP_sample_symbol_extended_info_list_t_val[ii].info.TSP_extended_info_list_t_val[0]), "unit", "e");
                        break; 

                    case 1005:
                        TSP_EIList_initialize(&(SSEI_list->TSP_sample_symbol_extended_info_list_t_val[ii].info),1);
                        TSP_EI_update(&(SSEI_list->TSP_sample_symbol_extended_info_list_t_val[ii].info.TSP_extended_info_list_t_val[0]), "unit", "f");
                        break; 

                    case 1006:
                        TSP_EIList_initialize(&(SSEI_list->TSP_sample_symbol_extended_info_list_t_val[ii].info),1);
                        TSP_EI_update(&(SSEI_list->TSP_sample_symbol_extended_info_list_t_val[ii].info.TSP_extended_info_list_t_val[0]), "unit", "g");
                        break; 

                    case 1007:
                        TSP_EIList_initialize(&(SSEI_list->TSP_sample_symbol_extended_info_list_t_val[ii].info),1);
                        TSP_EI_update(&(SSEI_list->TSP_sample_symbol_extended_info_list_t_val[ii].info.TSP_extended_info_list_t_val[0]), "unit", "h");
                        break; 
                    case 1008:
                        TSP_EIList_initialize(&(SSEI_list->TSP_sample_symbol_extended_info_list_t_val[ii].info),1);
                        TSP_EI_update(&(SSEI_list->TSP_sample_symbol_extended_info_list_t_val[ii].info.TSP_extended_info_list_t_val[0]), "unit", "i");
                        break; 

                    case 1009:
                        TSP_EIList_initialize(&(SSEI_list->TSP_sample_symbol_extended_info_list_t_val[ii].info),1);
                        TSP_EI_update(&(SSEI_list->TSP_sample_symbol_extended_info_list_t_val[ii].info.TSP_extended_info_list_t_val[0]), "unit", "j");
                        break; 

                    case 1010:
                        TSP_EIList_initialize(&(SSEI_list->TSP_sample_symbol_extended_info_list_t_val[ii].info),1);
                        TSP_EI_update(&(SSEI_list->TSP_sample_symbol_extended_info_list_t_val[ii].info.TSP_extended_info_list_t_val[0]), "unit", "m");
                        break; 

                    case 1011:
                        TSP_EIList_initialize(&(SSEI_list->TSP_sample_symbol_extended_info_list_t_val[ii].info),2);
                        TSP_EI_update(&(SSEI_list->TSP_sample_symbol_extended_info_list_t_val[ii].info.TSP_extended_info_list_t_val[0]), "profile", "10");
                        TSP_EI_update(&(SSEI_list->TSP_sample_symbol_extended_info_list_t_val[ii].info.TSP_extended_info_list_t_val[1]), "unit", "SI");
                        TSP_EI_update(&(SSEI_list->TSP_sample_symbol_extended_info_list_t_val[ii].info.TSP_extended_info_list_t_val[1]), "order", "1");
                        break; 

                    case 1012:
                        TSP_EIList_initialize(&(SSEI_list->TSP_sample_symbol_extended_info_list_t_val[ii].info),2);
                        TSP_EI_update(&(SSEI_list->TSP_sample_symbol_extended_info_list_t_val[ii].info.TSP_extended_info_list_t_val[0]), "profile", "2*10");
                        TSP_EI_update(&(SSEI_list->TSP_sample_symbol_extended_info_list_t_val[ii].info.TSP_extended_info_list_t_val[1]), "unit", "None");
                        TSP_EI_update(&(SSEI_list->TSP_sample_symbol_extended_info_list_t_val[ii].info.TSP_extended_info_list_t_val[1]), "order", "2");
                        break; 

                    default: // do not forget invalid > 1012 PGIs
                        SSEI_list->TSP_sample_symbol_extended_info_list_t_val[ii].provider_global_index = -1;      
                        SSEI_list->TSP_sample_symbol_extended_info_list_t_val[ii].info.TSP_extended_info_list_t_len = 0;
                        SSEI_list->TSP_sample_symbol_extended_info_list_t_val[ii].info.TSP_extended_info_list_t_val = NULL;
                        ret = TSP_STATUS_ERROR_PGI_UNKNOWN;     
                }
            }
        }
        else {
            SSEI_list->TSP_sample_symbol_extended_info_list_t_val[ii].provider_global_index = -1;      
            SSEI_list->TSP_sample_symbol_extended_info_list_t_val[ii].info.TSP_extended_info_list_t_len = 0;
            SSEI_list->TSP_sample_symbol_extended_info_list_t_val[ii].info.TSP_extended_info_list_t_val = NULL;
            ret = TSP_STATUS_ERROR_PGI_UNKNOWN;
        }
    }
    return  ret;
}


// ---------------------------------------------------------------------------------------------------
// tsp_provider_dll_initialize EXPORTED function
// ---------------------------------------------------------------------------------------------------
int32_t tsp_provider_dll_initialize(const char*	providerName, 
									double		baseFrequency)
{
    int  argc   = 1;
    char **argv = 0;
	
    // Create the glue stub
    GLU_handle_create(&GLU_stub, providerName, GLU_SERVER_TYPE_ACTIVE, baseFrequency);

    // Define the GLU functions
    GLU_stub->run                   = &STUB_GLU_thread;
    GLU_stub->initialize            = &STUB_GLU_init;
    GLU_stub->get_ssi_list          = &STUB_GLU_get_ssi_list;
    GLU_stub->get_ssei_list_fromPGI = &STUB_GLU_get_ssei_list_fromPGI;

    // Initialize TSP Provider library
    argv = (char**) calloc(argc + 1, sizeof(char *));
    argv[0] = strdup(providerName);
    argv[1] = NULL;
    if (TSP_STATUS_OK != TSP_provider_init(GLU_stub, &argc, &argv)) {
        return -1;
    }

    return 0;
}


// ---------------------------------------------------------------------------------------------------
// tsp_provider_dll_symbol_initialize EXPORTED function
// ---------------------------------------------------------------------------------------------------
int32_t tsp_provider_dll_symbol_initialize(int32_t nbSymbols)
{
    if (TSP_STATUS_OK!=TSP_SSIList_initialize(&X_SSI_list, nbSymbols)) {
        return -1;
    }

    // Store the max number of symbol
    nb_max_symbols = nbSymbols;

    return 0;
}


// ---------------------------------------------------------------------------------------------------
// tsp_provider_dll_symbol_add EXPORTED function
// ---------------------------------------------------------------------------------------------------
int32_t tsp_provider_dll_symbol_add(const char*	symbolName,								
									int32_t		symbolType,
									int32_t		dimension)
{
    int32_t                  size;
    TSP_sample_symbol_info_t *X_sample_symbol_info;

    if (nb_symbols >= nb_max_symbols) {
        return -1;
    }

    // Get the current sample infos
    X_sample_symbol_info = TSP_SSIList_getSSI(X_SSI_list, nb_symbols);

	// Update the  sample infos
    TSP_SSI_initialize(X_sample_symbol_info,
                       symbolName,  // name
                       nb_symbols,  // provider global index
                       0, 0,        // pgridx, pgrank
                       symbolType,  // type
                       dimension,   // dimension
                       0, 0,        // offset, nelem
                       1,           // period
                       0);          // phase

    // Update the symbol number and max size
    nb_symbols++;
    size = X_sample_symbol_info->dimension * tsp_type_size[X_sample_symbol_info->type];
    if (max_symbol_size < size) {
        max_symbol_size = size;
    }

    return nb_symbols;
}


// ---------------------------------------------------------------------------------------------------
// tsp_provider_dll_symbol_finalize EXPORTED function
// ---------------------------------------------------------------------------------------------------
int32_t tsp_provider_dll_symbol_finalize()
{
    // Alloc memory for the biggest symbol
    item.raw_value = calloc(1, max_symbol_size);
    assert(item.raw_value);

    // Start the provider
    if (TSP_STATUS_OK != TSP_provider_run(TSP_ASYNC_REQUEST_SIMPLE | TSP_ASYNC_REQUEST_NON_BLOCKING)) {
        return -1;
    }

    return 0;
}


// ---------------------------------------------------------------------------------------------------
// tsp_provider_dll_data_update EXPORTED function
// ---------------------------------------------------------------------------------------------------
int32_t tsp_provider_dll_data_update(int32_t	numSymbol,
									 double		symbolValue,
									 int32_t	size)
{
    TSP_sample_symbol_info_t	*X_sample_symbol_info;

    if (numSymbol >= nb_max_symbols) {
        return -1;
    }

    // Get the current symbol infos
    X_sample_symbol_info = TSP_SSIList_getSSI(X_SSI_list, numSymbol);

    // Update the current item infos
    item.provider_global_index = numSymbol;
    item.raw_value = &symbolValue;
    item.size = X_sample_symbol_info->dimension * tsp_type_size[X_sample_symbol_info->type];
    item.time = 0;

    // Push the current item in the datapool
    TSP_datapool_push_next_item(GLU_stub->datapool, &item);

    return 0;
}


// ---------------------------------------------------------------------------------------------------
// tsp_provider_dll_data_commit EXPORTED function
// ---------------------------------------------------------------------------------------------------
int32_t tsp_provider_dll_data_commit()
{
    TSP_datapool_push_commit(GLU_stub->datapool, 0, GLU_GET_NEW_ITEM);

    return 0;
}


// ---------------------------------------------------------------------------------------------------
// tsp_provider_dll_finalize EXPORTED function
// ---------------------------------------------------------------------------------------------------
int32_t tsp_provider_dll_finalize() 
{
    TSP_datapool_push_commit(GLU_stub->datapool, 0, GLU_GET_EOF);
    TSP_provider_end();

    return 0;
}
