//-----------------------------------------------------------------------
// A small TSP provider DLL sample.
//-----------------------------------------------------------------------
//
// Written by J-F. LAROCHE  and E. NOULARD
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
//
// -----------------------------------------------------------------------
//
// Project   : TSP
// Component : DLL Provider
// 
// -----------------------------------------------------------------------
// 
// Purpose   : A small TSP provider in the form of Windows DLL
//             Allow to define the number of symbol, the frequency,
//             adding symbol (only double are currently supported), 
//             and running the provider.
// 
// -----------------------------------------------------------------------


#ifndef  __TSP_PROVIDER_DLL_H
#define  __TSP_PROVIDER_DLL_H 1

#include <tsp_abs_types.h>

#if defined(WIN32)
#  ifdef tsp_provider_dll_EXPORTS
#    define _EXPORT_TSP_PROVIDER_DLL __declspec(dllexport) 
#  else
#    define _EXPORT_TSP_PROVIDER_DLL __declspec(dllimport) 
#  endif
#endif 

_EXPORT_TSP_PROVIDER_DLL
int32_t tsp_provider_dll_initialize(const char* providerName, 
                                    double      baseFrequency);

_EXPORT_TSP_PROVIDER_DLL
int32_t tsp_provider_dll_symbol_initialize(int32_t nbSymbols);

_EXPORT_TSP_PROVIDER_DLL
int32_t tsp_provider_dll_symbol_add(const char* symbolName,								
                                    int32_t     symbolType,
                                    int32_t     dimension);

_EXPORT_TSP_PROVIDER_DLL
int32_t tsp_provider_dll_symbol_finalize();

_EXPORT_TSP_PROVIDER_DLL
int32_t tsp_provider_dll_data_update(int32_t numSymbol,
                                     double  symbolValue,
                                     int32_t size);

_EXPORT_TSP_PROVIDER_DLL
int32_t tsp_provider_dll_data_commit();

_EXPORT_TSP_PROVIDER_DLL
int32_t tsp_provider_dll_finalize();

#endif /* __TSP_PROVIDER_DLL_H */